function lo = loewner_order_tf(X,Y)

% Loewner order of (1x1,) 2x2 or 3x3 Hermitian matrices X and Y: is X > Y?
%
% Loewner order: X > Y if (X-Y) is positive definite
%
% lo = loewner_order_tf(X,Y)
%
% SAR input images are images of the (row-wise) upper-right elements of
% the covariance or coherency matrix, e.g. for full polarimetry:
% a vector with nine real (i.e., non-complex) elements, namely
% [ShhShh* Re(ShhShv*) Im(ShhShv*) Re(ShhSvv*) Im(ShhSvv*) ...
%  ShvShv* Re(ShvSvv*) Im(ShvSvv*) SvvSvv*]
%
% Input
%   X       - 3-D array of (1x1,) 2x2 or 3x3 complex covariance matrix
%             (polarimetric SAR image in covariance matrix form,
%             reading order is alphabetical (as read by freadenvisar):
%               hhhh (real), hhhv (complex), hhvv (complex),
%                            hvhv (real),    hvvv (complex),
%                                            vvvv (real)
%   Y       - 3-D array of (1x1,) 2x2 or 3x3 complex covariance matrix
%
% Output
%   lo - 
%           lo(1) true	- X-Y is positive definite
%           lo(2) true	- X-Y is negative definite
%
% References
%
% Allan A. Nielsen, Henning Skriver and Knut Conradsen (2019).
% "The Loewner Order and Direction of Detected Change in Sentinel-1 and
% Radarsat-2 Data".
% IEEE Geoscience and Remote Sensing Letters 17(2), 242-246.
% DOI:10.1109/LGRS.2019.2918636
%
% Allan A. Nielsen (2020).
% "Fast matrix based computation of eigenvalues and the Loewner order
% in PolSAR data".
% IEEE Geoscience and Remote Sensing Letters 17(10), 1727-1731.
% DOI:10.1109/LGRS.2019.2952202

% (c) Copyright 2018-2023
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, people.compute.dtu.dk/alan
% 3-21 Sep 2018
% 13 Apr 2019 (fast, matrix based 2x2 and 3x3 azimuthal symmetry)
% 6-9 Jul 2019 (fast, matrix based full 3x3)
% 2 Aug 2019 (finally, pivot based - about bloody time, too!)
% 27 Nov 2023 /AA: two 3x3 diagonal blocks in 6x6 covmat
%                  two 2x2 diagonal blocks in 4x4 covmat

if nargin<2
    help loewner_order_tf
    error('loewner_order_tf: wrong input');
end
if nargout>1
    help loewner_order_tf
    error('loewner_order_tf: wrong output');
end

pX = size(X);
pY = size(Y);
if ~all(pX==pY)
    error('loewner_order_tf: X and Y must have same dimensions');
end
if  ndims(X)==3
    nvars = pX(3);
elseif ismatrix(X)
    warning('scalar input, no reason to use loewner_order_tf')
    nvars = 1;
else
    error('loewner_order_tf: wrong sizes of input images');
end

X = X-Y;
clear Y

if nvars==9 % full pol
    d1 = X(:,:,1);
    d2 = wishart_det(X(:,:,[1 2 3 6]));
    d3 = wishart_det(X);
    lo(1) = d1>0 & d2>0 & d3>0;
    lo(2) = d1<0 & d2>0 & d3<0;
elseif nvars==4 % dual pol
    d1 = X(:,:,1);
    d2 = wishart_det(X);
    lo(1) = d1>0 & d2>0;
    lo(2) = d1<0 & d2>0;
elseif nvars==2 % dual pol, diagonal only
    lo(1) = X(:,:,1)>0 & X(:,:,2)>0;
    lo(2) = X(:,:,1)<0 & X(:,:,2)<0;
elseif nvars==1 % single channel
    lo(1) = X>0;
    lo(2) = X<0;
elseif nvars==5 % full pol, azimuthal symmetry
    d1 = X(:,:,1);
    d2 = X(:,:,1).*X(:,:,4);
    d3 = wishart_det(X);
    lo(1) = d1>0 & d2>0 & d3>0;
    lo(2) = d1<0 & d2>0 & d3<0;
elseif nvars==3 % full pol, diagonal only
    lo(1) = X(:,:,1)>0 & X(:,:,2)>0 & X(:,:,3)>0;
    lo(2) = X(:,:,1)<0 & X(:,:,2)<0 & X(:,:,3)<0;
elseif nvars==18 % full pol, two 3x3 diagonal blocks in 6x6 covmat
    d1 = X(:,:,1);
    d2 = wishart_det(X(:,:,[1 2 3 6]));
    d3 = wishart_det(X(:,:,1:9));
    d1 = X(:,:,1+9).*d1;
    d2 = wishart_det(X(:,:,[1 2 3 6]+9)).*d2;
    d3 = wishart_det(X(:,:,(1:9)+9)).*d3;
    lo(1) = d1>0 & d2>0 & d3>0;
    lo(2) = d1<0 & d2>0 & d3<0;
elseif nvars==8 % dual pol, two 2x2 diagonal blocks in 4x4 covmat
    d1 = X(:,:,1);
    d2 = wishart_det(X(:,:,1:4));
    d1 = X(:,:,1+4).*d1;
    d2 = wishart_det(X(:,:,(1:4)+4)).*d2;
    lo(1) = d1>0 & d2>0;
    lo(2) = d1<0 & d2>0;
else
    error('loewner_order_tf: wrong dimensionality of input images')
end
