function [c,nsets] =  wc_when(wc,pval)

%  wc_when takes struct wc from wcRjl as input and outputs occurrence of change
%
% c =  wc_when(wc,0.9999);
%
% input wc is struct from wcRjl, pval is probability threshold for change prob, default 0.99
% output c(:,:,i) is array showing if lnQ and lnR shows change after time point i
%
% wcWhenRj detects change based on Rj only
% wc_when and wcWhenQRj detects change based on both Q and Rj

% (c) Copyright 2016-2019
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, http://people.compute.dtu.dk/alan
% 2 May 2016 - 30 Jan 2019

if ~isstruct(wc)
    error('wc must be struct as output from wcRjl')
end
if isfield(wc,'sumMat')
    nsets = size(wc,2);
else
    nsets = size(wc,2) + 1;
end
k = nsets;

if ismatrix(wc(nsets-1).P) % two time points only: copy lnQ to lnR
    wc(nsets-1).P(:,:,2) = wc(nsets-1).P;
    wc(nsets-1).P(:,:,1) = wc(nsets-1).P(:,:,2);
end

if nargin<2
    pval = 0.99;
end

sizec = size(wc(1).P);
sizec(3) = sizec(3) - 1;
c = false(sizec);

h = waitbar(0,'Processing...','Name','wc_when');
for cc=1:sizec(2)
    waitbar(cc/sizec(2),h)
    for rr=1:sizec(1)
        time = 0;
        ell = 1;
        while and(ell<k,time<k)
            if wc(ell).P(rr,cc,1)>pval % check Q
                for j=2:(k-ell+1)
                    time = time + 1;
                    if wc(ell).P(rr,cc,j)>pval % check Rjl
                        c(rr,cc,time) = true;
%                         fprintf('ell=%i, j=%i, time=%i, P=%f\n',ell,j,time,1-wc(ell).P(rr,cc,j))
                        ell = ell + j - 1;
                        break;
                    end
                end
            else
                break;
            end
        end
    end
end
close(h)

return

% Change visualizations

% RGB examples
caux = single(c(:,:,1:3));
% caux = single(c(:,:,end-2:end));
% caux = single(c(:,:,[1 3 5]));
% caux = modefilter(single(caux));
figure('Name','RGB example'), imshow(caux), axis off
% clear caux

% change at time point 2 and on with no previous changes
figure, imshow(c(:,:,1))
for k = 2:(nsets-1)%(nsets-1):-1:2
    figure, imshow(~any(c(:,:,1:k-1),3) & c(:,:,k))
end
% % as in:
% figure, imshow(~c(:,:,1) &  c(:,:,2))
% figure, imshow(~c(:,:,1) & ~c(:,:,2) &  c(:,:,3))
% figure, imshow(~c(:,:,1) & ~c(:,:,2) & ~c(:,:,3) &  c(:,:,4))
% figure, imshow(~c(:,:,1) & ~c(:,:,2) & ~c(:,:,3) & ~c(:,:,4) & c(:,:,5))

% first change
[maxch,change1] = max(c,[],3);
change1(maxch==0) = 0;
figure('Name','First change'), imshow(change1,[]), axis off
colormap(gca,jet0)
h = colorbar('southoutside');
set(h,'XTick',0:(nsets-1),'Position',[0.10 0.08 0.80 0.025]) % [left bottom width height]
% set(h,'XTick',0:(nsets-1),'Position',[0.22 0.91 0.7 0.025],'Color','White')

set(h,'XTick',0:(nsets-1))
% set(h,'XTick',h.Limits(1):h.Limits(2))

% most recent/last change
[maxch,changel] = max(c(:,:,(nsets-1):-1:1),[],3);
changel = nsets - changel; 
changel(maxch==0) = 0;
figure('Name','Last change'), imshow(changel,[]), axis off
colormap(gca,jet0)
h = colorbar('southoutside');
set(h,'XTick',0:(nsets-1),'Position',[0.10 0.08 0.80 0.025]) % [left bottom width height]
% set(h,'XTick',0:(nsets-1))
% % set(h,'XTick',h.Limits(1):h.Limits(2))

% # of changes
sumc = sum(c,3);
figure('Name','#changes'), imshow(sumc,[]), axis off
colormap(gca,jet0)
h = colorbar('southoutside');
set(h,'XTick',0:(nsets-1),'Position',[0.10 0.08 0.80 0.025]) % [left bottom width height]
% set(h,'XTick',0:(nsets-1))
% set(h,'XTick',h.Limits(1):h.Limits(2))
% sumcmode = modefilter(sumc);
% figure
% imshow(sumcmode,[])
% colormap(gca,jet0)
% h = colorbar;
% set(h,'XTick',0:(nsets-1))
% % set(h,'XTick',h.Limits(1):h.Limits(2))
